/* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this
* file, You can obtain one at https://mozilla.org/MPL/2.0/.
*/
/** @file */
/** @copyright 2021 MPLv2 */
#include <Whisper/Message.hpp>
#include <Whisper/Payload.hpp>
#include <random>

std::uint64_t Whisper::Message::randomID_()
{
  std::random_device rd;
  std::mt19937 rng(rd());

  std::uniform_int_distribution<std::mt19937::result_type> dist(1,std::numeric_limits<std::uint64_t>::max());

  return dist(rng);
}


std::shared_ptr<Tree::BaseNode> Whisper::Message::getPayload()
{

  if (payload_ == nullptr)
  {
    throw std::runtime_error("no payload set yet");
  }

  std::shared_ptr<Tree::TreeRoot> payload = std::make_shared<Tree::TreeRoot>();
  payload->addChild(std::make_shared<Whisper::Payload<std::uint8_t>>("type",(std::uint8_t)type_));
  payload->addChild(std::make_shared<Whisper::Payload<std::uint64_t>>("src",srcID_));
  payload->addChild(std::make_shared<Whisper::Payload<std::uint64_t>>("dst",dstID_));
  payload->addChild(std::make_shared<Whisper::Payload<std::uint64_t>>("messageID",messageID_));
  payload->addChild(std::make_shared<Whisper::Payload<std::uint64_t>>("replyID",replyID_));
  payload->addChild(payload_->getPayload());


  return payload;
}


std::shared_ptr<std::vector<std::uint8_t>> Whisper::Message::encode()
{
  if (!hasEnconding())
  {
    throw std::runtime_error("no encoding set yet");
  }

  if (payload_ == nullptr)
  {
    throw std::runtime_error("no payload set yet");
  }

  // set the encoding for our payload
  payload_->setEncoding(encoding());


}

void Whisper::Message::decode(std::shared_ptr<std::vector<std::uint8_t>> data)
{

}

std::uint64_t Whisper::Message::size()
{

}
